@@ -57,42 +57,12 @@ extern "C" {
 #define SEQPATTERNSIZE 128
 #define CHARSIZE sizeof(widechar)
 #define DEFAULTRULESIZE 50
-#define ENDSEGMENT 0xffff
-
-/**
- * Definitions of braille dots
- */
-#define B1 0X01
-#define B2 0X02
-#define B3 0X04
-#define B4 0X08
-#define B5 0X10
-#define B6 0X20
-#define B7 0X40
-#define B8 0X80
-#define B9 0X100
-#define B10 0X200
-#define B11 0X400
-#define B12 0X800
-#define B13 0X1000
-#define B14 0X2000
-#define B15 0X4000
-#define B16 0X8000
 
 typedef struct intCharTupple {
 	int key;
 	char value;
 } intCharTupple;
 
-/**
- * Mapping between braille dot and textual representation as used in dots operands
- */
-static const intCharTupple dotMapping[] = {
-	{ B1, '1' }, { B2, '2' }, { B3, '3' }, { B4, '4' }, { B5, '5' }, { B6, '6' },
-	{ B7, '7' }, { B8, '8' }, { B9, '9' }, { B10, 'A' }, { B11, 'B' }, { B12, 'C' },
-	{ B13, 'D' }, { B14, 'E' }, { B15, 'F' }, { 0, 0 },
-};
-
 /* HASHNUM must be prime */
 #define HASHNUM 1123
 
@@ -130,15 +100,16 @@ typedef enum {
 	CTC_UserDefined6 = 0x400000,
 	CTC_UserDefined7 = 0x800000,  // class 12
 	CTC_CapsMode = 0x1000000,
-	CTC_NumericMode = 0x2000000,
-	CTC_NumericNoContract = 0x4000000,
-	CTC_EndOfInput = 0x8000000,  // only used by pattern matcher
-	CTC_EmpMatch = 0x10000000,   // only used in TranslationTableRule->before and
-								 // TranslationTableRule->after
-	CTC_MidEndNumericMode = 0x20000000,
-	// two more bits available in a unsigned int of width 32
-	// currently used for class 13 and 14
-	CTC_Class13 = 0x40000000,
+	CTC_EmphMode = 0x2000000,
+	CTC_NumericMode = 0x4000000,
+	CTC_NumericNoContract = 0x8000000,
+	CTC_EndOfInput = 0x10000000,  // only used by pattern matcher
+	CTC_EmpMatch = 0x20000000,	// only used in TranslationTableRule->before and
+								  // TranslationTableRule->after
+	CTC_MidEndNumericMode = 0x40000000,
+	// 33 more bits available in a unsigned long long (at least 64 bits)
+	// currently used for classes 13 to 45
+	CTC_Class13 = 0x80000000,
 } TranslationTableCharacterAttribute;
 
 typedef enum {
@@ -208,7 +179,7 @@ typedef enum {
 	pass_all = 255
 } pass_Codes;
 
-typedef unsigned int TranslationTableCharacterAttributes;
+typedef unsigned long long TranslationTableCharacterAttributes;
 
 typedef struct {
 	TranslationTableOffset next;
@@ -267,7 +238,7 @@ typedef enum { /* Op codes */
 	/* End of ordered opcodes */
 
 	CTO_CapsModeChars,
-	// CTO_EmphModeChars,
+	CTO_EmphModeChars,
 	CTO_BegComp,
 	CTO_CompBegEmph1,
 	CTO_CompEndEmph1,
@@ -494,6 +465,28 @@ typedef struct /* one state */
 	widechar numTrans;
 } HyphenationState;
 
+typedef struct CharacterClass {
+	struct CharacterClass *next;
+	TranslationTableCharacterAttributes attribute;
+	widechar length;
+	widechar name[1];
+} CharacterClass;
+
+typedef struct RuleName {
+	struct RuleName *next;
+	TranslationTableOffset ruleOffset;
+	widechar length;
+	widechar name[1];
+} RuleName;
+
+typedef struct {
+	TranslationTableOffset tableSize;
+	TranslationTableOffset bytesUsed;
+	TranslationTableOffset charToDots[HASHNUM];
+	TranslationTableOffset dotsToChar[HASHNUM];
+	TranslationTableOffset ruleArea[1]; /** Space for storing all rules and values */
+} DisplayTableHeader;
+
 /**
  * Translation table header
  */
@@ -504,6 +497,7 @@ typedef struct { /* translation table */
 	int syllables;
 	int usesSequences;
 	int usesNumericMode;
+	int usesEmphMode;
 	TranslationTableOffset tableSize;
 	TranslationTableOffset bytesUsed;
 	TranslationTableOffset undefined;
@@ -519,6 +513,11 @@ typedef struct { /* translation table */
 	/* emphRules, including caps. */
 	TranslationTableOffset emphRules[MAX_EMPH_CLASSES + 1][9];
 
+	/* state needed during compilation */
+	CharacterClass *characterClasses;
+	TranslationTableCharacterAttributes nextCharacterClassAttribute;
+	RuleName *ruleNames;
+
 	TranslationTableOffset begComp;
 	TranslationTableOffset compBegEmph1;
 	TranslationTableOffset compEndEmph1;
@@ -539,8 +538,6 @@ typedef struct { /* translation table */
 	int noLetsignAfterCount;
 	TranslationTableOffset characters[HASHNUM]; /** Character definitions */
 	TranslationTableOffset dots[HASHNUM];		/** Dot definitions */
-	TranslationTableOffset charToDots[HASHNUM];
-	TranslationTableOffset dotsToChar[HASHNUM];
 	TranslationTableOffset compdotsPattern[256];
 	TranslationTableOffset swapDefinitions[NUMSWAPS];
 	TranslationTableOffset forPassRules[MAXPASS + 1];
@@ -639,22 +636,36 @@ _lou_resolveTable(const char *tableList, const char *base);
 char **EXPORT_CALL
 _lou_defaultTableResolver(const char *tableList, const char *base);
 
-char *EXPORT_CALL
-_lou_getLastTableList(void);
-
 /**
  * Return single-cell dot pattern corresponding to a character.
  * TODO: move to commonTranslationFunctions.c
  */
 widechar EXPORT_CALL
-_lou_getDotsForChar(widechar c);
+_lou_getDotsForChar(widechar c, const DisplayTableHeader *table);
 
 /**
  * Return character corresponding to a single-cell dot pattern.
  * TODO: move to commonTranslationFunctions.c
  */
 widechar EXPORT_CALL
-_lou_getCharFromDots(widechar d);
+_lou_getCharFromDots(widechar d, const DisplayTableHeader *table);
+
+void EXPORT_CALL
+_lou_getTable(const char *tableList, const char *displayTableList,
+		const TranslationTableHeader **translationTable,
+		const DisplayTableHeader **displayTable);
+
+const TranslationTableHeader *EXPORT_CALL
+_lou_getTranslationTable(const char *tableList);
+
+const DisplayTableHeader *EXPORT_CALL
+_lou_getDisplayTable(const char *tableList);
+
+int EXPORT_CALL
+_lou_compileTranslationRule(const char *tableList, const char *inString);
+
+int EXPORT_CALL
+_lou_compileDisplayRule(const char *tableList, const char *inString);
 
 /**
  * Allocate memory for internal buffers
@@ -668,34 +679,43 @@ _lou_allocMem(AllocBuf buffer, int index, int srcmax, int destmax);
 
 /**
  * Hash function for character strings
+ *
+ * @param lowercase Whether to convert the string to lowercase because
+ *                  making the hash of it.
  */
-int EXPORT_CALL
-_lou_stringHash(const widechar *c);
+unsigned long int EXPORT_CALL
+_lou_stringHash(const widechar *c, int lowercase, const TranslationTableHeader *table);
 
 /**
  * Hash function for single characters
  */
-int EXPORT_CALL
+unsigned long int EXPORT_CALL
 _lou_charHash(widechar c);
 
 /**
  * Return a string in the same format as the characters operand in opcodes
- * TODO: move to utils.c
  */
-char *EXPORT_CALL
-_lou_showString(widechar const *chars, int length);
+const char *EXPORT_CALL
+_lou_showString(widechar const *chars, int length, int forceHex);
+
+/**
+ * Print out dot numbers
+ *
+ * @return a string containing the dot numbers. The longest possible
+ * output is "\123456789ABCDEF0/"
+ */
+const char *EXPORT_CALL
+_lou_unknownDots(widechar dots);
 
 /**
  * Return a character string in the format of the dots operand
- * TODO: move to utils.c
  */
-char *EXPORT_CALL
+const char *EXPORT_CALL
 _lou_showDots(widechar const *dots, int length);
 
 /**
  * Return a character string where the attributes are indicated
  * by the attribute letters used in multipass opcodes
- * TODO: move to utils.c
  */
 char *EXPORT_CALL
 _lou_showAttributes(TranslationTableCharacterAttributes a);
@@ -742,16 +762,16 @@ int EXPORT_CALL
 _lou_extParseDots(const char *inString, widechar *outString);
 
 int EXPORT_CALL
-_lou_translateWithTracing(const char *tableList, const widechar *inbuf, int *inlen,
-		widechar *outbuf, int *outlen, formtype *typeform, char *spacing, int *outputPos,
-		int *inputPos, int *cursorPos, int mode, const TranslationTableRule **rules,
-		int *rulesLen);
+_lou_translate(const char *tableList, const char *displayTableList, const widechar *inbuf,
+		int *inlen, widechar *outbuf, int *outlen, formtype *typeform, char *spacing,
+		int *outputPos, int *inputPos, int *cursorPos, int mode,
+		const TranslationTableRule **rules, int *rulesLen);
 
 int EXPORT_CALL
-_lou_backTranslateWithTracing(const char *tableList, const widechar *inbuf, int *inlen,
-		widechar *outbuf, int *outlen, formtype *typeform, char *spacing, int *outputPos,
-		int *inputPos, int *cursorPos, int mode, const TranslationTableRule **rules,
-		int *rulesLen);
+_lou_backTranslate(const char *tableList, const char *displayTableList,
+		const widechar *inbuf, int *inlen, widechar *outbuf, int *outlen,
+		formtype *typeform, char *spacing, int *outputPos, int *inputPos, int *cursorPos,
+		int mode, const TranslationTableRule **rules, int *rulesLen);
 
 void EXPORT_CALL
 _lou_resetPassVariables(void);
@@ -808,6 +828,17 @@ extern int translation_direction;
 int EXPORT_CALL
 _lou_isValidMode(int mode);
 
+/**
+ * Return the default braille representation for a character.
+ */
+widechar EXPORT_CALL
+_lou_charToFallbackDots(widechar c);
+
+static inline int
+isASCII(widechar c) {
+	return (c >= 0X20) && (c < 0X7F);
+}
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
